function InitPassword() {
    this.hostname = window.location.hostname;
}

InitPassword.prototype = {
    initialize: function() {
        this.getApplicationConfiguration()
            .then(this.onConfigSuccess.bind(this))
            .catch(this.onConfigError.bind(this));
    },

    /**
     * Read the user's configuration file
     * @returns a promise resolved with the configuration file content as an object
     */
    getApplicationConfiguration: function() {
        var thiz = this;
        return new Promise(function(resolve, reject) {
            var xhr = new XMLHttpRequest();
            xhr.addEventListener('readystatechange', function() {
                if(this.readyState === 4) {
                    if (xhr.status === 200) {
                        try {
                            var response = JSON.parse(this.responseText);
                            thiz.checkConfigValidity(response);
                            resolve(response);
                        } catch(e) {
                            reject(e);
                        }
                    } else {
                        reject('Invalid HTTP response status ' + xhr.status);
                    }
                }
            });
            console.log('Getting accounts-setup.json (you can edit this file with your settings)...');
            xhr.open('GET', 'accounts-setup.json');
            xhr.send();
        });
    },

    /**
     * Check the configuration validity, throwing a string describing the error
     * if needed.
     * @param {object} config
     */
    checkConfigValidity: function(config) {
        /**
         * Deep find an attribute.
         * @param {object} config is the root object
         * @param {string[]} keys the path to the attribute
         * @returns the parent of the attribute or undefined
         */
        function getAttribute(config, keys) {
            var source = config;
            for (var i = 0 ; source !== undefined && i < keys.length; i++) {
                if (source.hasOwnProperty(keys[i])) {
                    source = source[keys[i]];
                } else {
                    source = undefined;
                }
            }
            return source;
        }

        /**
         * Check that an attribute is present and is a non empty string
         * @param {object} config is the root object
         * @param {string[]} keys the path to the attribute
         */
        function checkMandatoryString(config, keys) {
            var attribute = getAttribute(config, keys);
            if (attribute === undefined) {
                throw "Missing attribute " + keys.join('.') + ".";
            }

            checkOptionalString(config, keys);
        }

        /**
         * Check that an optional attribute is a non empty string
         * @param {object} config is the root object
         * @param {string[]} keys the path to the attribute
         */
        function checkOptionalString(config, keys) {
            var attribute = getAttribute(config, keys);

            if (attribute !== undefined) {
                if (typeof attribute !== 'string') {
                    throw "Attribute " + keys.join('.') + " must be a string.";
                } else if (!attribute) {
                    throw "Attribute " + keys.join('.') + " must not be an empty string.";
                }
            }
        }

        checkMandatoryString(config, ["admin-password"]);
        checkOptionalString(config, ["httpserver", "admin"]);
        checkOptionalString(config, ["httpserver", "user"]);
        checkOptionalString(config, ["ftpserver", "username"]);
        checkOptionalString(config, ["ftpserver", "password"]);
    },

    /**
     * Process the configuration file, executing the requested actions
     * @param {object} config
     */
    onConfigSuccess: function(config) {
        var adminpasswd = config['admin-password'];

        this.client = IDAL.getClient(this.hostname, 'admin', adminpasswd);

        /**
         * Set an HTTP password and update the UI accordingly
         * @param {string} id is the HTMLElement identifier base
         * @param {string} username is the name of the user to set password
         * @returns {Promise<boolean>} an always resolved promise
         */
        var processHttpServerPassword = function(id, username) {
            if (config.httpserver && config.httpserver.hasOwnProperty(username)) {
                var promise = this.setHttpPassword(adminpasswd, username, config.httpserver[username]);
                return this.setPromiseState(promise, id);
            } else {
                this.setUIState(id, 'not-available');
                return Promise.resolve(false);
            }
        }.bind(this);


        /**
         * Get an attribute of the ftpserver, setting the UI with not available
         * if not available
         * @param {string} id is the HTMLElement identifier base
         * @param {string} attribute is the name of the attribute
         * @returns {string} the attribute value or undefined
         */
        var getFtpServerAttribute = function(id, attribute) {
            if (config.ftpserver) {
                if  (config.ftpserver.hasOwnProperty(attribute)) {
                    return config.ftpserver[attribute];
                } else {
                    this.setUIState(id, 'not-available');
                }
            } else {
                this.setUIState(id, 'not-available');
            }
        }.bind(this);

        /**
         * Send a request to set FTP server settings, updating the UI with
         * the request result.
         * @param {string} id is the HTMLElement identifier base
         * @param {object} ftpserver are the ftpserver attribute of the set system setting request
         * @returns {Promise<boolean>} an always resolved promise
         */
        var sendFtpServerSettings = function(id, ftpserver) {
            /*
             * Process the request success.
             * Beware, a success can hide errors...
             * Returns an always resolved promise.
             */
            var onSuccess = function(result) {
                if (result.errors && result.errors[0]) {
                    this.setUIState(id, 'error', result.errors[0].message);
                    return Promise.resolve(false);
                } else {
                    this.setUIState(id, 'success');
                    return Promise.resolve(true);
                }
            }.bind(this);

            /**
             * Process a request error.
             * Returns an always resolved promise.
             */
            var onError = function(error) {
                var message;
                if (error.errors && error.errors[0]) {
                    message = error.errors[0].message;
                }
                this.setUIState(id, 'error', message);
                return Promise.resolve(false);
            }.bind(this);

            return this.client.put("/api/system/settings", { ftpserver: ftpserver })
                .then(onSuccess)
                .catch(onError);
        }.bind(this);


        /**
         * Let's work...
         * Chain executions so that setting the admin password is
         * always the last request
         */
        Promise.resolve(true)
            .then(function() {
                var ftpUser = getFtpServerAttribute('ftp-server-username', 'username');
                if (ftpUser) {
                    return sendFtpServerSettings('ftp-server-username', {
                        username: ftpUser
                    });
                }
                return Promise.resolve(false);
            })
            .then(function() {
                var ftpPassword = getFtpServerAttribute('ftp-server-password', 'password');
                if (ftpPassword) {
                    return sendFtpServerSettings('ftp-server-password', {
                        password: this.encodeFtpPassword(ftpPassword)
                    });
                }
                return Promise.resolve(false);
            }.bind(this))
            .then(function() {
                return processHttpServerPassword('http-server-user-password', 'user');
            })
            .then(function() {
                return processHttpServerPassword('http-server-admin-password', 'admin');
            });
    },

    /**
     * Error handler for configuration fetching/checking/processing.
     * Update the UI accordingly.
     * @param {*} error
     */
    onConfigError: function(error) {
        var div = document.getElementById('error-config');
        if (div) {
            div.classList.remove('d-none');
        }

        var detail = document.getElementById('error-config-detail');
        if (detail) {
            detail.innerText = error;
        }

        this.setUIState('ftp-server-username', 'error');
        this.setUIState('ftp-server-password', 'error');
        this.setUIState('http-server-admin-password', 'error');
        this.setUIState('http-server-user-password', 'error');
    },

    /**
     * Set the state of the UI for a given action/request
     * @param {string} id is the HTMLElement identifier base.
     * @param {'success'|'error'|'not-available'} status is the status to display
     * @param {*} info are the error information to display.
     */
    setUIState: function(id, status, info) {
        var icon = document.getElementById("icon-" + id);
        if (icon) {
            if (status == 'success') {
                icon.setAttribute('class', 'fa fa-check-circle success');
            } else if (status == 'error') {
                icon.setAttribute('class', 'fa fa-exclamation-circle error');
            } else {
                icon.setAttribute('class', 'fa fa-ban not-available');
            }
        }

        if (info) {
            var details = document.getElementById("details-" + id);
            if (details) {
                if (typeof info !== 'string') {
                    details.innerText = JSON.stringify(info);
                } else {
                    details.innerText = info;
                }

            }
        }
    },

    /**
     * Set the the state of an action (http request) on the user interface
     * @param {Promise<*>} promise the promise to watch
     * @param {string} id HTMLElement identifier to modify
     * @returns {Promise<boolean>} an always resolved promise
     */
    setPromiseState: function(promise, id) {
        var onSuccess = function(result) {
            if (result && result.errors && result.errors[0]) {
                this.setUIState(id, 'error', result.errors[0].message);
                return false;
            } else {
                this.setUIState(id, 'success');
                return true;
            }
        }.bind(this);

        var onError = function(error) {
            var message;
            if (error && error.errors && error.errors[0]) {
                message = error.errors[0].message;
            }

            this.setUIState(id, 'error', message);
            return false;
        }.bind(this);

        return promise
            .then(onSuccess)
            .catch(onError);
    },

    /**
     * Set an HTTP server password
     * @param {string} adminpasswd is the administrator password
     * @param {string} username is the name of the user whose password must be set
     * @param {string} password is the password for the user referenced by username
     * @returns
     */
    setHttpPassword: function(adminpasswd, username, password) {
        var ha1 = CryptoJS.MD5(['admin', 'Authentication required', adminpasswd].join(':')).toString().toLowerCase();
        var encodedPassword = CryptoJS.AES.encrypt(password, ha1).toString();

        var payload = {
            username,
            password: encodedPassword,
            signature: CryptoJS.SHA1(ha1 + ':' + username + ':' + encodedPassword).toString().toLowerCase()
        }

        return this.client.post('/api/auth/set-password', payload);
    },

    encodeFtpPassword: function(password) {
        var MAX_SEED = 99999999;
        var HASH_ITERATIONS = 1000;

        const salt = (Math.random() * MAX_SEED).toFixed().toString();
        var hash = salt + password;

        for (var i = 0 ; i < HASH_ITERATIONS ; i++) {
            hash = CryptoJS.MD5(hash).toString().toUpperCase();
        }
        return salt + ':' + hash;
    }
};


var gApp;

window.addEventListener('load', function() {
    gApp = new InitPassword();
    gApp.initialize();
});
